#SFTP Master
import os, tarfile, paramiko
from stat import S_ISDIR


class SFTApi(object):

    transport = None
    sftp = None
    zipname = None

    def __init__(self, *args, **kwargs):
        self.hostname = kwargs['FTP_DOMAIN']
        self.port = kwargs['FTP_PORT']
        self.username = kwargs['FTP_USERNAME']
        self.password = kwargs['FTP_PASSWORD']
        self._connect()

    def _connect(self, *args, **kwargs):
        """ It is used to connect with remote host """
        self._get_ssh()
        self.transport = paramiko.Transport((self.hostname, self.port))
        self.transport.connect(username = self.username, password = self.password)
        
        self.sftp = paramiko.SFTPClient.from_transport(self.transport)
        print "Connected to ", self.hostname, "as ", self.username
        return

    def _get_ssh(self, *args, **kwargs):
        """ It is used to connect as ssh """
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(self.hostname, username = self.username, password = self.password)
        return ssh

    
    def isExists(self, path):
        """ It will return whether a file/directory exists or not and return type """
        """ 1 - directory, 0 - file, -1 - Not exists """
        is_file = os.path.isfile(path)
        is_dir = os.path.isdir(path)

        if is_file or is_dir:
            if is_file:
                return 0
            else:
                return 1
        else:
            return -1

    def remove(self, path):
        self.sftp.remove(path)


    def upload(self, *args, **kwargs):
        """ It is used to upload files / folders """
        if "source" and "destination" not in kwargs:
            return
        else:
            path_type = self.isExists(kwargs['source'])
            if path_type == -1:
                return 
            if path_type == 0:
                #Copy directly
                file_name = kwargs['source'].split("/")[len(kwargs['source'].split("/")) -1]
                self.sftp.put(kwargs['source'], kwargs['destination'] + file_name)
            if path_type == 1:
                #  recursively upload a full directory
                os.chdir(os.path.split(kwargs['source'])[0])
                parent=os.path.split(kwargs['source'])[1]
                for walker in os.walk(parent):
                    try:
                        self.sftp.mkdir(os.path.join(kwargs['destination'],walker[0]))
                    except:
                        pass
                    for file in walker[2]:
                        self.sftp.put(os.path.join(walker[0],file),os.path.join(kwargs['destination'],walker[0],file))
        return 
    def sftp_walk(self,remotepath):
        # Kindof a stripped down  version of os.walk, implemented for 
        # sftp.  Tried running it flat without the yields, but it really
        # chokes on big directories.
        path = remotepath
        files = []
        folders = []
        for f in self.sftp.listdir_attr(remotepath):
            if S_ISDIR(f.st_mode):
                folders.append(f.filename)
            else:
                files.append(f.filename)
        yield path,folders,files
        for folder in folders:
            new_path=os.path.join(remotepath,folder)
            for x in self.sftp_walk(new_path):
                yield x


    def download(self, *args, **kwargs):
        """ It is used to download the file / folders """
        if "source" and "destination" not in kwargs:
            return
        else:
            if 'type' not in kwargs or kwargs['type'] == "File":
                self.sftp.get(kwargs['source'], kwargs['destination'] + kwargs['source'].split("/")[-1])
            else:
                self.sftp.chdir(os.path.split(kwargs['source'])[0])
                parent = os.path.split(kwargs['source'])[1]
                try:
                    os.mkdir(kwargs['destination'])
                except:
                    pass
                for walker in self.sftp_walk(parent):
                    try:
                        os.mkdir(os.path.join(kwargs['destination'],walker[0]))
                    except:
                        pass
                    for file in walker[2]:
                        
                        try:
                            self.sftp.get(os.path.join(walker[0],file),os.path.join(kwargs['destination'],walker[0],file))
                        except:
                            pass
        return

    



from setuptools import setup

setup(
  name = 'sftp_s3_sync',
  packages = ['sftp_s3_sync'], # this must be the same as the name above
  version = '2.0.1',
  description = 'A python sftp and s3 syncing library',
  author = 'Partha',
  author_email = 'parthasaradhi1992@gmail.com',
  url = 'https://github.com/parthz/sftp-s3-sync', 
  download_url = 'https://github.com/parthz/sftp-s3-sync/tarball/0.1', 
  keywords = ['ftp sync s3', 'sftp sync s3'], 
  classifiers = [],
  install_requires=[
        "boto",
        "paramiko"
    ],
  zip_safe=False)



#Author: Partha

import os, shutil, tempfile
from sftp_api import SFTApi
from boto.s3.connection import S3Connection,Key

class SyncFtpS3(object):

    sftp = None

    def __init__(self, *args, **kwargs):
        ## Initialize remote ftp/sftp connection
        self.sftp = SFTApi(FTP_DOMAIN = kwargs['FTP_DOMAIN'],
                           FTP_PORT = kwargs['FTP_PORT'],
                           FTP_USERNAME = kwargs['FTP_USERNAME'], 
                           FTP_PASSWORD = kwargs['FTP_PASSWORD'])
        
        ## Initialize s3 connection
        self.AWS_ACCESS_KEY_ID = kwargs['AWS_ACCESS_KEY_ID']
        self.AWS_SECRET_ACCESS_KEY = kwargs['AWS_SECRET_ACCESS_KEY']
        self.S3_BUCKET = kwargs['S3_BUCKET']
        self._aws_s3()

    def _aws_s3(self):
        """ Connect to s3 with the given credentials """
        try:
            conn = S3Connection(self.AWS_ACCESS_KEY_ID, self.AWS_SECRET_ACCESS_KEY)
            self._bucket = conn.get_bucket(self.S3_BUCKET)
        except:
            print "Connection Error"


    def _downloadFromFtp(self, *args, **kwargs):
        """ It will download the contents from FTP Server and will place into _temp_location """
        self.sftp.download(type = kwargs['type'], 
            source = kwargs['source'], 
            destination = kwargs['destination'] + "/")

    def _uploadToS3(self, local_path, to_file_path, to_file_name, s3_path):
        """ It will upload each file into s3 """
        try:
            to_file_path = to_file_path.replace(local_path, "")
            if to_file_path.endswith("/"):
                to_file_path = to_file_path + "/"
            ## Write to s3            
            k = Key(self._bucket)
            k.key = s3_path + to_file_path + '/' + to_file_name
            f = open(local_path + to_file_path + "/" + to_file_name, 'r')
            k.set_contents_from_string(f.read())
            f.close()
        except:
            print "Fail:", local_path,to_file_path,to_file_name

    def sync(self, source = None, destination = None, _type = None):
        ## Temparory directory to work with
        self._temp_location = tempfile.mkdtemp()

        ## 1. Download files from ftp server to local_server
        self._downloadFromFtp(type = _type, 
            source = source, 
            destination = self._temp_location)
        
        ## 2. File by file upload to the s3 and then cloud
        for subdir, dirs, files in os.walk(self._temp_location):
            for file in files:
                self._uploadToS3(self._temp_location, subdir, file, destination)
        
        #3. Remove temperaroy directory & recreate with empty content
        try:
            shutil.rmtree(self._temp_location)
        except:
            pass

        



